@@ -128,26 +128,6 @@ field2: value22
 field3: value23
 '
 
-test_declare_input_file compound-names \
-'%rec: Hacker
-
-Name: John Smith
-Email: john@smith.net
-
-Name: Jose E. Marchesi
-Email: jemarch@gnu.org
-
-%rec: Task
-
-Id: 1
-Summary: This is task 1
-Hacker:OpenedBy: John Smith
-
-Id: 2
-Summary: This is task 2
-Hacker:OpenedBy: Jose E. Marchesi
-'
-
 test_declare_input_file dates \
 'Date: Tue Nov 30 12:00:00 CET 2002
 
@@ -197,6 +177,51 @@ Id: -2
 Key: baz
 '
 
+test_declare_input_file sort-with-comment \
+'%rec: Sorted
+%sort: Id
+%type: Id int
+
+# Foo
+
+# Bar
+
+Id: 100
+Key: foo
+
+Id: 20
+Key: bar
+
+Id: -2
+Key: baz
+'
+
+test_declare_input_file sort-multiple \
+'%rec: SortMultiple
+%sort: Class Price
+%type: Price real
+
+Item: one
+Class: B
+Price: 30
+
+Item: two
+Class: A
+Price: 30
+
+Item: three
+Class: C
+Price: 40
+
+Item: four
+Class: A
+Price: 10
+
+Item: five
+Class: B
+Price: 15
+'
+
 test_declare_input_file empty-field-values \
 'a: a1
 b:
@@ -222,6 +247,153 @@ field1: foo
 field1: foo
 '
 
+test_declare_input_file group-records \
+'id: 3
+pos: 1
+
+id: 1
+pos: 2
+
+id: 1
+pos: 3
+
+id: 2
+pos: 4
+
+id: 3
+pos: 5
+'
+
+test_declare_input_file group-records-missing \
+'id: 3
+pos: 1
+
+id: 1
+pos: 2
+
+pos: 3
+
+id: 2
+pos: 4
+
+id: 3
+pos: 5
+'
+
+test_declare_input_file sales \
+'Item: A
+Date: 21 April 2012
+Cost: 23
+
+Item: B
+Date: 20 April 2012
+Cost: 50
+
+Item: A
+Date: 21 April 2012
+Cost: 10
+
+Item: C
+Date: 12 January 2003
+Cost: 12
+
+Item: D
+Date: 20 April 2012
+Cost: 100
+'
+
+test_declare_input_file packages-maintainers \
+'%rec: Package
+%type: Maintainer,PreviousMaintainer rec Hacker
+
+Name: GNU PDF
+URL: http://www.gnupdf.org
+Maintainer: jemarch@gnu.org
+Maintainer: aleksander@gnu.org
+
+Name: GNU recutils
+URL: http://www.gnu.org/software/recutils
+Maintainer: jemarch@gnu.org
+
+Name: GNU Emacs
+URL: http://www.gnu.org/software/emacs
+Maintainer: stefan@monnier.com
+PreviousMaintainer: rms@gnu.org
+
+%rec: Hacker
+%key: Email
+%type: Email email
+
+Name: Jose E. Marchesi
+Email: jemarch@gnu.org
+URL: http://www.jemarch.net
+
+Name: Aleksander Morgado
+Email: aleksander@gnu.org
+
+Name: Richard M. Stallman
+Email: rms@gnu.org
+URL: http://www.stallman.org
+
+Name: Stefan Monnier
+Email: stefan@monnier.com
+'
+
+test_declare_input_file implications \
+'%rec: Task
+
+Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 1
+Status: Closed
+
+Id: 2
+Status: Open
+'
+
+test_declare_input_file non-existant-foreign-key \
+'%rec: Record
+%type: Foreign rec NonExistant
+
+Name: Fred
+Foreign: xxx
+'
+
+test_declare_input_file several-foreign-keys \
+'%rec: R
+%key: Id
+
+Id: R1
+Description: this is R1
+
+Id: R2
+Description: this is R1
+
+Id: R3
+Description: this is R1
+
+%rec: T
+%key: Id
+%type: Requirement rec R
+
+Id: T1
+Requirement: R1
+
+Id: T2
+Requirement: R2
+Requirement: R3
+
+Id: T3
+Requirement: R3
+'
+
+test_declare_input_file unquoted-lisp-strings \
+'foo: fo\o
+bar: a quote"etouq a
+'
+
 #
 # Declare tests
 #
@@ -280,6 +452,16 @@ test_tool recsel-values ok \
 
 value22
 
+value32
+'
+test_tool recsel-values-long ok \
+          recsel \
+          '--print-values=field2' \
+          multiple-records \
+'value12
+
+value22
+
 value32
 '
 
@@ -333,6 +515,17 @@ value21 value22
 value31 value32
 '
 
+test_tool recsel-row-long ok \
+          recsel \
+          '--print-row=field1,field2' \
+          multiple-records \
+'value11 value12
+
+value21 value22
+
+value31 value32
+'
+
 # Print collapsed in a row
 test_tool recsel-collapsed-row ok \
           recsel \
@@ -427,6 +620,15 @@ field2: value12
 field3: value13
 '
 
+test_tool recsel-index-long ok \
+          recsel \
+          '--number=0' \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+'
+
 test_tool recsel-index-second ok \
           recsel \
           '-n 1' \
@@ -558,24 +760,6 @@ field3: value33
 
 # Selection expressions.
 
-test_tool recsel-sex-field-names ok \
-          recsel \
-          '-t Task -e "OpenedBy = '\''John Smith'\''"' \
-          compound-names \
-'Id: 1
-Summary: This is task 1
-Hacker:OpenedBy: John Smith
-'
-
-test_tool recsel-sex-field-names-2 ok \
-          recsel \
-          '-t Task -e "Hacker:OpenedBy: = '\''Jose E. Marchesi'\''"' \
-          compound-names \
-'Id: 2
-Summary: This is task 2
-Hacker:OpenedBy: Jose E. Marchesi
-'
-
 test_tool recsel-sex-integer-equal ok \
           recsel \
           '-e "field1 = 0"' \
@@ -615,6 +799,15 @@ test_tool recsel-sex-integer-lessthan ok \
 'field1: -10
 '
 
+test_tool recsel-sex-integer-lessequalthan ok \
+          recsel \
+          '-e "field1 <= 0"' \
+          integer-fields \
+'field1: -10
+
+field1: 0
+'
+
 test_tool recsel-sex-integer-biggerthan ok \
           recsel \
           '-e "field1 > 10"' \
@@ -622,6 +815,15 @@ test_tool recsel-sex-integer-biggerthan ok \
 'field1: 314
 '
 
+test_tool recsel-sex-integer-biggerequalthan ok \
+          recsel \
+          '-e "field1 >= 10"' \
+          integer-fields \
+'field1: 314
+
+field1: 10
+'
+
 test_tool recsel-sex-integer-plus ok \
           recsel \
           '-e "field1 + 2 = 316"' \
@@ -712,7 +914,6 @@ test_tool recsel-sex-real-lessthan ok \
           real-fields \
 'field1: -10.0
 '
-
 test_tool recsel-sex-real-biggerthan ok \
           recsel \
           '-e "field1 > 3.14"' \
@@ -766,11 +967,22 @@ field2: value121
 field2: value122
 '
 
+test_tool recsel-sex-sharp-subname ok \
+          recsel \
+          '-t Package -j Maintainer -e "#Maintainer.URL" -P Maintainer.URL' \
+          packages-maintainers \
+'http://www.jemarch.net
+
+http://www.jemarch.net
+'
+
 test_tool recsel-sex-match ok \
           recsel \
-          '-t Hacker -p Name -e "Email ~ '\''\\.org'\''"' \
-          compound-names \
-'Name: Jose E. Marchesi
+          '-p Name -e "Name ~ '\''Tom'\''"' \
+          academy \
+'Name: Tom Johnson
+
+Name: Tommy Junior
 '
 
 test_tool recsel-sex-date-sametime ok \
@@ -798,6 +1010,18 @@ test_tool recsel-sex-date-after ok \
 Date: Tue Nov 30 12:00:00 CET 2030
 '
 
+test_tool recsel-sex-implies-1 ok \
+          recsel \
+          '-e "Status = '\''Closed'\'' => #ClosedBy"' \
+          implications \
+'Id: 0
+Status: Closed
+ClosedBy: jemarch
+
+Id: 2
+Status: Open
+'
+
 test_tool recsel-sex-conditional-1 ok \
           recsel \
           '-e "Role ~ '\''Professor'\'' ? Age > 65 : Age < 10" -p Name' \
@@ -843,6 +1067,50 @@ bar"\""' -c" \
 '1
 '
 
+test_tool recsel-sex-string-equal ok \
+          recsel \
+          "-e 'field2 = \"value12\"'" \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+'
+
+test_tool recsel-sex-string-non-equal ok \
+          recsel \
+          "-e 'field2 != \"value12\"'" \
+          multiple-records \
+'field1: value21
+field2: value22
+field3: value23
+
+field1: value31
+field2: value32
+field3: value33
+'
+
+test_tool recsel-sex-string-equal-insensitive ok \
+          recsel \
+          "-i -e 'field2 = \"vaLue12\"'" \
+          multiple-records \
+'field1: value11
+field2: value12
+field3: value13
+'
+
+test_tool recsel-sex-string-non-equal-insensitive ok \
+          recsel \
+          "-i -e 'field2 != \"vaLue12\"'" \
+          multiple-records \
+'field1: value21
+field2: value22
+field3: value23
+
+field1: value31
+field2: value32
+field3: value33
+'
+
 test_tool recsel-quick-simple ok \
           recsel \
           "-q value22" \
@@ -941,6 +1209,45 @@ Id: 100
 Key: foo
 '
 
+test_tool recsel-sort-with-comment ok \
+          recsel \
+          '' \
+          sort-with-comment \
+'Id: -2
+Key: baz
+
+Id: 20
+Key: bar
+
+Id: 100
+Key: foo
+'
+
+test_tool recsel-sort-multiple ok \
+          recsel \
+          '' \
+          sort-multiple \
+'Item: four
+Class: A
+Price: 10
+
+Item: two
+Class: A
+Price: 30
+
+Item: five
+Class: B
+Price: 15
+
+Item: one
+Class: B
+Price: 30
+
+Item: three
+Class: C
+Price: 40
+'
+
 test_tool recsel-sort-field ok \
           recsel \
           '-S Key' \
@@ -955,6 +1262,30 @@ Id: 100
 Key: foo
 '
 
+test_tool recsel-sort-field-multiple ok \
+          recsel \
+          '-S Price,Class' \
+          sort-multiple \
+'Item: four
+Class: A
+Price: 10
+
+Item: five
+Class: B
+Price: 15
+
+Item: two
+Class: A
+Price: 30
+
+Item: one
+Class: B
+Price: 30
+
+Item: three
+Class: C
+Price: 40
+'
 test_tool recsel-sort-field-nonexist ok \
           recsel \
           '-S doesnotexist' \
@@ -1041,6 +1372,392 @@ field2: value2
 field3: value3
 '
 
+test_tool recsel-group-records ok \
+          recsel \
+          '-G id' \
+          group-records \
+'id: 1
+pos: 2
+pos: 3
+
+id: 2
+pos: 4
+
+id: 3
+pos: 1
+pos: 5
+'
+
+test_tool recsel-group-records-sort ok \
+          recsel \
+          '-G pos' \
+          group-records \
+'id: 3
+pos: 1
+
+id: 1
+pos: 2
+
+id: 1
+pos: 3
+
+id: 2
+pos: 4
+
+id: 3
+pos: 5
+'
+
+test_tool recsel-group-records-non-existing-field ok \
+          recsel \
+          '-G doesnotexist' \
+          group-records \
+'id: 3
+pos: 1
+
+id: 1
+pos: 2
+
+id: 1
+pos: 3
+
+id: 2
+pos: 4
+
+id: 3
+pos: 5
+'
+
+test_tool recsel-group-one-record ok \
+          recsel \
+          '-G field3' \
+          one-record \
+'field1: value1
+field2: value2
+field3: value3
+'
+
+test_tool recsel-group-records-missing ok \
+          recsel \
+          '-G id' \
+          group-records-missing \
+'pos: 3
+
+id: 1
+pos: 2
+
+id: 2
+pos: 4
+
+id: 3
+pos: 1
+pos: 5
+'
+
+test_tool recsel-group-multiple-fields ok \
+          recsel \
+          '-G Date,Item' \
+          sales \
+'Item: C
+Date: 12 January 2003
+Cost: 12
+
+Item: B
+Date: 20 April 2012
+Cost: 50
+
+Item: D
+Date: 20 April 2012
+Cost: 100
+
+Item: A
+Date: 21 April 2012
+Cost: 23
+Cost: 10
+'
+
+# rewrite rule
+
+test_tool recsel-aggregate-field-name ok \
+           recsel \
+           '-p "Count(Cost)"' \
+           sales \
+'Count_Cost: 5
+'
+
+test_tool recsel-aggregate-case-insensitive ok \
+          recsel \
+          '-p "cOuNt(Cost)"' \
+          sales \
+'cOuNt_Cost: 5
+'
+
+test_tool recsel-aggregate-rewrite-rule ok \
+          recsel \
+          '-p "Count(Cost):MyCount"' \
+          sales \
+'MyCount: 5
+'
+
+test_tool recsel-aggregate-count-overall ok \
+          recsel \
+          '-P "Count(Cost)"' \
+          sales \
+'5
+'
+
+test_tool recsel-aggregate-count-grouped ok \
+          recsel \
+          '-p "Item,Count(Cost)" -G Item' \
+          sales \
+'Item: A
+Count_Cost: 2
+
+Item: B
+Count_Cost: 1
+
+Item: C
+Count_Cost: 1
+
+Item: D
+Count_Cost: 1
+'
+
+test_tool recsel-aggregate-avg-overall ok \
+          recsel \
+          '-P "Avg(Cost)"' \
+          sales \
+'39
+'
+
+test_tool recsel-aggregate-avg-grouped ok \
+          recsel \
+          '-p "Item,Avg(Cost)" -G Item' \
+          sales \
+'Item: A
+Avg_Cost: 16.500000
+
+Item: B
+Avg_Cost: 50
+
+Item: C
+Avg_Cost: 12
+
+Item: D
+Avg_Cost: 100
+'
+
+test_tool recsel-aggregate-sum-overall ok \
+          recsel \
+          '-P "Sum(Cost)"' \
+          sales \
+'195
+'
+
+test_tool recsel-aggregate-sum-grouped ok \
+          recsel \
+          '-p "Item,Sum(Cost):TotalCost" -G Item' \
+          sales \
+'Item: A
+TotalCost: 33
+
+Item: B
+TotalCost: 50
+
+Item: C
+TotalCost: 12
+
+Item: D
+TotalCost: 100
+'
+
+test_tool recsel-aggregate-min-overall ok \
+          recsel \
+          '-P "Min(Cost)"' \
+          sales \
+'10
+'
+
+test_tool recsel-aggregate-min-grouped ok \
+          recsel \
+          '-p "Item,Min(Cost)" -G Item' \
+          sales \
+'Item: A
+Min_Cost: 10
+
+Item: B
+Min_Cost: 50
+
+Item: C
+Min_Cost: 12
+
+Item: D
+Min_Cost: 100
+'
+
+test_tool recsel-aggregate-max-overall ok \
+          recsel \
+          '-P "Max(Cost)"' \
+          sales \
+'100
+'
+
+test_tool recsel-aggregate-max-grouped ok \
+          recsel \
+          '-p "Item,Max(Cost)" -G Item' \
+          sales \
+'Item: A
+Max_Cost: 23
+
+Item: B
+Max_Cost: 50
+
+Item: C
+Max_Cost: 12
+
+Item: D
+Max_Cost: 100
+'
+
+test_tool recsel-fex-rewrite-all ok \
+          recsel \
+          "-p field1,field2:xxx,field3" \
+          repeated-fields \
+'field1: value11
+xxx: value121
+xxx: value122
+field3: value13
+
+field1: value21
+xxx: value221
+xxx: value222
+field3: value23
+'
+
+test_tool recsel-fex-rewrite-subscript-single ok \
+          recsel \
+          "-p field1,field2[0]:xxx,field2[1],field3" \
+          repeated-fields \
+'field1: value11
+xxx: value121
+field2: value122
+field3: value13
+
+field1: value21
+xxx: value221
+field2: value222
+field3: value23
+'
+test_tool recsel-fex-rewrite-subscript-multi ok \
+          recsel \
+          "-p field1,field2[0-1]:xxx,field3" \
+          repeated-fields \
+'field1: value11
+xxx: value121
+xxx: value122
+field3: value13
+
+field1: value21
+xxx: value221
+xxx: value222
+field3: value23
+'
+
+test_tool recsel-fex-rewrite-several ok \
+          recsel \
+          "-p field1,field2:xxx,field3:yyy" \
+          repeated-fields \
+'field1: value11
+xxx: value121
+xxx: value122
+yyy: value13
+
+field1: value21
+xxx: value221
+xxx: value222
+yyy: value23
+'
+
+test_tool recsel-descriptor ok \
+          recsel \
+          "-t type2 -d" \
+          multiple-types \
+'%rec: type2
+
+field1: value21
+field2: value22
+field3: value23
+'
+
+test_tool recsel-nonexistent-print ok \
+          recsel \
+          '-p dontexist' \
+          multiple-records \
+          ''
+
+test_tool recsel-join-default-rset xfail \
+          recsel \
+          '-j field2' \
+          multiple-records
+
+test_tool recsel-join-no-foreign ok \
+          recsel \
+          '-t type1 -j field2' \
+          multiple-types \
+'field1: value11
+field2: value12
+field3: value13
+'
+
+test_tool recsel-non-existant-foreign-key ok \
+          recsel \
+          '-t Record -j Foreign' \
+          non-existant-foreign-key \
+'Name: Fred
+Foreign: xxx
+'
+
+test_tool recsel-several-foreign-keys ok \
+          recsel \
+          '-t T -j Requirement -p Id,Requirement.Id' \
+          several-foreign-keys \
+'Id: T1
+Requirement_Id: R1
+
+Id: T2
+Requirement_Id: R2
+
+Id: T2
+Requirement_Id: R3
+
+Id: T3
+Requirement_Id: R3
+'
+
+test_tool recsel-join-descriptor ok \
+          recsel \
+          '-t Package -j Maintainer -d -p Name' \
+         packages-maintainers \
+'%rec: Package_Maintainer
+
+Name: GNU PDF
+
+Name: GNU PDF
+
+Name: GNU recutils
+
+Name: GNU Emacs
+'
+
+test_tool recsel-unquoted-lisp-strings ok \
+          recsel \
+          '--print-sexps' \
+          unquoted-lisp-strings \
+'(record 0 (
+(field 0 "foo" "fo\\o")
+(field 11 "bar" "a quote\"etouq a")))
+'
+
 #
 # Cleanup
 #
