@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,36 +45,23 @@
 #include <regex.h>
 #include <stdint.h>
 #include <time.h>
+#include <unistd.h>
+#include <getpass.h>
 
 #include <rec.h>
 #include <recutl.h>
-
-#define MAX_INDEXES 20
-
-struct recutl_index_s
-{
-  size_t min;
-  size_t max;
-  bool max_used;
-};
-
-struct recutl_index_list_s
-{
-  size_t size;
-  struct recutl_index_s *indexes;
-};
-
-typedef struct recutl_index_list_s recutl_index_list_t;
+#include "read-file.h"
 
 /*
  * Global variables.
  */
 
-bool              recutl_sort_p         = false;
-char             *recutl_order_rset     = NULL;
-rec_field_name_t  recutl_order_by_field = NULL;
-bool              recutl_interactive_p  = false;
-recutl_index_list_t recutl_indexes;
+static bool    recutl_sort_p         = false;
+static char   *recutl_order_rset     = NULL;
+static char   *recutl_order_by_field = NULL;
+static bool    recutl_interactive_p  = false;
+static size_t *recutl_indexes        = NULL;
+static size_t  recutl_indexes_size   = 0;
 
 void recutl_print_help (void); /* Forward prototype.  */
 
@@ -108,8 +95,8 @@ recutl_init (char *util_name)
   recutl_interactive_p = isatty (fileno(stdin));
 
   /* Initially there are no indexes.  */
-  
-  recutl_indexes.size = 0;
+
+  recutl_reset_indexes ();
 }
 
 bool
@@ -163,10 +150,10 @@ Record selection options:\n\
   -i, --case-insensitive              make strings case-insensitive in selection\n\
                                         expressions.\n\
   -t, --type=TYPE                     operate on records of the specified type only.\n\
-  -e, --expression=EXPR               selection expression.\n\
+  -e, --expression=RECORD_EXPR        selection expression.\n\
   -q, --quick=STR                     select records with fields containing a string.\n\
   -n, --number=NUM,...                select specific records by position, with ranges.\n\
-  -R, --random=NUM                    select a given number of random records.\n"),
+  -m, --random=NUM                    select a given number of random records.\n"),
          stdout);
 }
 
@@ -208,6 +195,12 @@ recutl_fatal (const char *fmt, ...)
   exit (EXIT_FAILURE);
 }
 
+void
+recutl_out_of_memory (void)
+{
+  recutl_fatal (_("out of memory\n"));
+}
+
 void
 recutl_error (const char *fmt, ...)
 {
@@ -269,7 +262,8 @@ recutl_parse_db_from_file (FILE *in,
       rec_parser_perror (parser, "%s", file_name);
       res = false;
     }
-  
+  rec_parser_destroy (parser);
+
   return res;
 }
 
@@ -281,9 +275,18 @@ recutl_build_db (int argc, char **argv)
   FILE *in;
 
   db = rec_db_new ();
+  if (!db)
+    {
+      return NULL;
+    }
+
+  /* Register the default functions in the database.  */
+
+  
 
   /* Process the input files, if any.  Otherwise use the standard
      input to read the rec data.  */
+
   if (optind < argc)
     {
       while (optind < argc)
@@ -317,39 +320,6 @@ recutl_build_db (int argc, char **argv)
   return db;
 }
 
-char *
-recutl_eval_field_expression (rec_fex_t fex,
-                              rec_record_t record,
-                              rec_writer_mode_t mode,
-                              bool print_values_p,
-                              bool print_in_a_row_p,
-                              char *password)
-{
-  char *res;
-  size_t res_size;
-  rec_writer_t writer;
-  bool wrote_p;
-
-  writer = rec_writer_new_str (&res, &res_size);
-  if (password)
-    {
-      rec_writer_set_password (writer,
-                               password);
-    }
-
-  wrote_p = rec_write_record_with_fex (writer, record, fex, mode,
-                                       print_values_p, print_in_a_row_p);
-  rec_writer_destroy (writer);
-
-  if (!wrote_p)
-    {
-      free (res);
-      res = NULL;
-    }
-
-  return res;
-}
-
 rec_db_t
 recutl_read_db_from_file (char *file_name)
 {
@@ -383,12 +353,18 @@ recutl_read_db_from_file (char *file_name)
   return db;
 }
 
+bool
+recutl_file_is_writable (char *file_name)
+{
+  return !file_name || (euidaccess (file_name, W_OK) == 0);
+}
+
 void
 recutl_write_db_to_file (rec_db_t db,
                          char *file_name)
 {
   FILE *out;
-  char *tmp_file_name;
+  char tmp_file_name[10]="recXXXXXX";
   rec_writer_t writer;
   int des;
   struct stat st1;
@@ -404,8 +380,6 @@ recutl_write_db_to_file (rec_db_t db,
       stat_result = stat (file_name, &st1);
 
       /* Create a temporary file with the results. */
-      tmp_file_name = xmalloc (100);
-      strcpy (tmp_file_name, "recXXXXXX");
       des = mkstemp (tmp_file_name);
       if (des == -1)
         {
@@ -446,26 +420,14 @@ recutl_read_file (char *file_name)
 {
   char *result;
   FILE *in;
-  size_t file_size;
 
   result = NULL;
   in = fopen (file_name, "r");
   if (in)
     {
-      /* Get the size of the file.  */
-      fseek (in, 0, SEEK_END);
-      file_size = ftell (in);
-      fseek (in, 0, SEEK_SET);
-
-      /* Read the contents of the file into file_name.  */
-      result = xmalloc (file_size + 1);
-      if (fread (result, file_size, 1, in) == 0)
-        {
-          recutl_fatal (_("reading file %s"), file_name);
-        }
-
+      size_t file_size;
+      result = fread_file (in, &file_size);
       fclose (in);
-      result[file_size] = '\0';
     }
 
   return result;
@@ -538,12 +500,17 @@ recutl_index_list_parse (const char *str)
   const char *p;
   long int number;
   char *end;
+  size_t i;
 
   /* Initialize the list structure.   An pessimistic estimation of the
      number of indexes encoded in the string is used.  */
 
-  recutl_indexes.size = 0;
-  recutl_indexes.indexes = xmalloc (sizeof (struct recutl_index_s) * strlen (str));
+  free (recutl_indexes);
+  recutl_indexes = xmalloc (sizeof (size_t) * (strlen (str) * 2 + 2));
+  for (i = 0; i < (strlen (str) * 2 + 2); i++)
+    {
+      recutl_indexes[i] = REC_Q_NOINDEX;
+    }
   
   /* Make sure the string is valid.  The code below relies on this
      fact.  */
@@ -570,7 +537,7 @@ recutl_index_list_parse (const char *str)
       /* Get the 'min' part of the entry.  */
 
       number = strtol (p, &end, 10);
-      recutl_indexes.indexes[recutl_indexes.size].min = (size_t) number;
+      recutl_indexes[recutl_indexes_size] = (size_t) number;
       p = end;
 
       /* Get the 'max' part of the entry, if any.  */
@@ -579,17 +546,11 @@ recutl_index_list_parse (const char *str)
         {
           p++;
           number = strtol (p, &end, 10);
-          recutl_indexes.indexes[recutl_indexes.size].max = (size_t) number;
+          recutl_indexes[recutl_indexes_size+1] = (size_t) number;
           p = end;
-
-          recutl_indexes.indexes[recutl_indexes.size].max_used = true;
-        }
-      else
-        {
-          recutl_indexes.indexes[recutl_indexes.size].max_used = false;
         }
 
-      recutl_indexes.size++;
+      recutl_indexes_size = recutl_indexes_size + 2;
 
       /* Exit or pass the separator.  */
 
@@ -606,90 +567,57 @@ recutl_index_list_parse (const char *str)
   return res;
 }
 
-size_t
-recutl_num_indexes (void)
-{
-  return recutl_indexes.size;
-}
-
-bool
-recutl_index_p (size_t index)
-{
-  bool res = false;
-  size_t i;
-
-  for (i = 0; i < recutl_indexes.size; i++)
-    {
-      size_t min = recutl_indexes.indexes[i].min;
-
-      if (recutl_indexes.indexes[i].max_used)
-        {
-          size_t max = recutl_indexes.indexes[i].max;
-          res = ((index >= min) && (index <= max));
-        }
-      else
-        {
-          res = (index == min);
-        }
-
-      if (res)
-        {
-          break;
-        }
-    }
-  
-  return res;
-}
-
-void
-recutl_index_add_random (size_t num, size_t limit)
-{
-  size_t i;
-  char random_state[128];
-  struct random_data random_data;
-
-  /* Initialize the list structure.  */
-
-  recutl_indexes.size = num;
-  recutl_indexes.indexes = xmalloc (sizeof (struct recutl_index_s) * num);
-
-  /* Insert the random indexes.  */
-
-  memset (&random_data, 0, sizeof (random_data));
-  initstate_r (time(NULL), (char *) &random_state, 128, &random_data);
-  for (i = 0; i < num; i++)
-    {
-      int32_t random_value = 0;
-
-      random_r (&random_data, &random_value); /* Can't fail.  */
-      random_value = random_value % limit;
-
-      if (recutl_index_p (random_value))
-        {
-          /* Pick the first available number.  */
-
-          size_t i;
-          for (i = 0; i < limit; i++)
-            {
-              if (!recutl_index_p (i))
-                {
-                  random_value = i;
-                  break;
-                }
-            }
-        }
-
-      recutl_indexes.indexes[i].min = random_value;
-      recutl_indexes.indexes[i].max = 0;
-      recutl_indexes.indexes[i].max_used = false;
-    }
-}
-
 void
 recutl_reset_indexes (void)
 {
-  free (recutl_indexes.indexes);
-  recutl_indexes.size = 0;
+  free (recutl_indexes);
+  recutl_indexes = NULL;
+  recutl_indexes_size = 0;
+}
+
+size_t
+recutl_num_indexes (void)
+{
+  return recutl_indexes_size;
+}
+
+size_t *
+recutl_index (void)
+{
+  return recutl_indexes;
+}
+
+char *
+recutl_getpass (bool asktwice)
+{
+  char *ret = NULL;
+
+  char *pass = getpass (_("Password: "));
+  if (pass)
+    {
+      ret = xstrdup (pass);
+      if (asktwice)
+        {
+          pass = getpass (_("Password again: "));
+          if (pass)
+            {
+              if (strcmp (ret, pass) != 0)
+                {
+                  recutl_fatal (_("the provided passwords don't match.\n"));
+                  memset (ret, 0, strlen (ret));
+                  memset (pass, 0, strlen (pass));
+                }
+            }
+          else
+            {
+              memset (ret, 0, strlen (ret));
+              free (ret);
+              ret = NULL;
+            }
+        }
+    }
+
+  return ret;
 }
 
 /* End of recutl.c */
